package java.util.zip;

import java.util.NoSuchElementException;
import java.io.EOFException;
import sun.misc.SharedSecrets;
import sun.misc.JavaUtilZipFileAccess;
import sun.misc.VM;
import java.util.Iterator;
import java.util.HashMap;
import java.util.stream.StreamSupport;
import java.util.Spliterators;
import java.util.stream.Stream;
import java.util.Enumeration;
import sun.misc.PerfCounter;
import java.util.ArrayDeque;
import java.util.WeakHashMap;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.io.IOException;
import java.io.File;
import java.util.Deque;
import java.io.InputStream;
import java.util.Map;
import java.io.Closeable;

public class ZipFile implements ZipConstants, Closeable
{
    private long jzfile;
    private final String name;
    private final int total;
    private final boolean locsig;
    private volatile boolean closeRequested;
    private static final int STORED = 0;
    private static final int DEFLATED = 8;
    public static final int OPEN_READ = 1;
    public static final int OPEN_DELETE = 4;
    private static final boolean usemmap;
    private static final boolean ensuretrailingslash;
    private ZipCoder zc;
    private final Map<InputStream, Inflater> streams;
    private Deque<Inflater> inflaterCache;
    private static final int JZENTRY_NAME = 0;
    private static final int JZENTRY_EXTRA = 1;
    private static final int JZENTRY_COMMENT = 2;
    
    private static native void initIDs();
    
    public ZipFile(String pathname) throws IOException {
        this(new File(pathname), 1);
    }
    
    public ZipFile(File file, int n) throws IOException {
        this(file, n, StandardCharsets.UTF_8);
    }
    
    public ZipFile(File file) throws ZipException, IOException {
        this(file, 1);
    }
    
    public ZipFile(File file, int i, Charset charset) throws IOException {
        this.closeRequested = false;
        this.streams = new WeakHashMap();
        this.inflaterCache = new ArrayDeque();
        if ((i & 0x1) == 0x0 || (i & 0xFFFFFFFA) != 0x0) {
            throw new IllegalArgumentException("Illegal mode: 0x" + Integer.toHexString(i));
        }
        String path = file.getPath();
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkRead(path);
            if ((i & 0x4) != 0x0) {
                securityManager.checkDelete(path);
            }
        }
        if (charset == null) {
            throw new NullPointerException("charset is null");
        }
        this.zc = ZipCoder.get(charset);
        long nanoTime = System.nanoTime();
        this.jzfile = open(path, i, file.lastModified(), ZipFile.usemmap);
        PerfCounter.getZipFileOpenTime().addElapsedTimeFrom(nanoTime);
        PerfCounter.getZipFileCount().increment();
        this.name = path;
        this.total = getTotal(this.jzfile);
        this.locsig = startsWithLOC(this.jzfile);
        return;
    }
    
    public ZipFile(String pathname, Charset charset) throws IOException {
        this(new File(pathname), 1, charset);
    }
    
    public ZipFile(File file, Charset charset) throws IOException {
        this(file, 1, charset);
    }
    
    public String getComment() {
        synchronized (this) {
            this.ensureOpen();
            byte[] commentBytes = getCommentBytes(this.jzfile);
            if (commentBytes == null) {
                return null;
            }
            return this.zc.toString(commentBytes, commentBytes.length);
        }
    }
    
    public ZipEntry getEntry(String s) {
        if (s == null) {
            throw new NullPointerException("name");
        }
        synchronized (this) {
            this.ensureOpen();
            long entry = getEntry(this.jzfile, this.zc.getBytes(s), true);
            if (entry != 0L) {
                ZipEntry zipEntry = ZipFile.ensuretrailingslash ? this.getZipEntry(null, entry) : this.getZipEntry(s, entry);
                freeEntry(this.jzfile, entry);
                return zipEntry;
            }
        }
        return null;
    }
    
    private static native long getEntry(long p0, byte[] p1, boolean p2);
    
    private static native void freeEntry(long p0, long p1);
    
    public InputStream getInputStream(ZipEntry zipEntry) throws IOException {
        if (zipEntry == null) {
            throw new NullPointerException("entry");
        }
        synchronized (this) {
            this.ensureOpen();
            long n;
            if (!this.zc.isUTF8() && (zipEntry.flag & 0x800) != 0x0) {
                n = getEntry(this.jzfile, this.zc.getBytesUTF8(zipEntry.name), false);
            }
            else {
                n = getEntry(this.jzfile, this.zc.getBytes(zipEntry.name), false);
            }
            if (n == 0L) {
                return null;
            }
            ZipFile.ZipFileInputStream o = new ZipFile.ZipFileInputStream(n);
            switch (getEntryMethod(n)) {
                case 0: {
                    synchronized (this.streams) {
                        this.streams.put(o, null);
                    }
                    return o;
                }
                case 8: {
                    long n2 = getEntrySize(n) + 2L;
                    if (n2 > 65536L) {
                        n2 = 8192L;
                    }
                    if (n2 <= 0L) {
                        n2 = 4096L;
                    }
                    Inflater inflater = this.getInflater();
                    ZipFile.ZipFileInflaterInputStream o2 = new ZipFile.ZipFileInflaterInputStream(o, inflater, (int)n2);
                    synchronized (this.streams) {
                        this.streams.put(o2, inflater);
                    }
                    return o2;
                }
                default: {
                    throw new ZipException("invalid compression method");
                }
            }
        }
    }
    
    private Inflater getInflater() {
        synchronized (this.inflaterCache) {
            Inflater inflater;
            while (null != (inflater = (Inflater)this.inflaterCache.poll())) {
                if (inflater.ended()) {
                    continue;
                }
                return inflater;
            }
        }
        return new Inflater(true);
    }
    
    private void releaseInflater(Inflater inflater) {
        if (!inflater.ended()) {
            inflater.reset();
            synchronized (this.inflaterCache) {
                this.inflaterCache.add(inflater);
            }
        }
    }
    
    public String getName() {
        return this.name;
    }
    
    public Enumeration<? extends ZipEntry> entries() {
        return new ZipFile.ZipEntryIterator();
    }
    
    public Stream<? extends ZipEntry> stream() {
        return StreamSupport.stream(Spliterators.spliterator(new ZipFile.ZipEntryIterator(), (long)this.size(), 1297), false);
    }
    
    private ZipEntry getZipEntry(String name, long n) {
        ZipEntry zipEntry = new ZipEntry();
        zipEntry.flag = getEntryFlag(n);
        if (name != null) {
            zipEntry.name = name;
        }
        else {
            byte[] entryBytes = getEntryBytes(n, 0);
            if (entryBytes == null) {
                zipEntry.name = "";
            }
            else if (!this.zc.isUTF8() && (zipEntry.flag & 0x800) != 0x0) {
                zipEntry.name = this.zc.toStringUTF8(entryBytes, entryBytes.length);
            }
            else {
                zipEntry.name = this.zc.toString(entryBytes, entryBytes.length);
            }
        }
        zipEntry.xdostime = getEntryTime(n);
        zipEntry.crc = getEntryCrc(n);
        zipEntry.size = getEntrySize(n);
        zipEntry.csize = getEntryCSize(n);
        zipEntry.method = getEntryMethod(n);
        zipEntry.setExtra0(getEntryBytes(n, 1), false);
        byte[] entryBytes2 = getEntryBytes(n, 2);
        if (entryBytes2 == null) {
            zipEntry.comment = null;
        }
        else if (!this.zc.isUTF8() && (zipEntry.flag & 0x800) != 0x0) {
            zipEntry.comment = this.zc.toStringUTF8(entryBytes2, entryBytes2.length);
        }
        else {
            zipEntry.comment = this.zc.toString(entryBytes2, entryBytes2.length);
        }
        return zipEntry;
    }
    
    private static native long getNextEntry(long p0, int p1);
    
    public int size() {
        this.ensureOpen();
        return this.total;
    }
    
    @Override
    public void close() throws IOException {
        if (this.closeRequested) {
            return;
        }
        this.closeRequested = true;
        synchronized (this) {
            synchronized (this.streams) {
                if (!this.streams.isEmpty()) {
                    HashMap hashMap = new HashMap(this.streams);
                    this.streams.clear();
                    for (Map.Entry entry : hashMap.entrySet()) {
                        ((InputStream)entry.getKey()).close();
                        Inflater inflater = (Inflater)entry.getValue();
                        if (inflater != null) {
                            inflater.end();
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
            synchronized (this.inflaterCache) {
                Inflater inflater2;
                while (null != (inflater2 = (Inflater)this.inflaterCache.poll())) {
                    inflater2.end();
                }
            }
            if (this.jzfile != 0L) {
                long jzfile = this.jzfile;
                this.jzfile = 0L;
                close(jzfile);
            }
        }
        return;
    }
    
    @Override
    protected void finalize() throws IOException {
        this.close();
    }
    
    private static native void close(long p0);
    
    private void ensureOpen() {
        if (this.closeRequested) {
            throw new IllegalStateException("zip file closed");
        }
        if (this.jzfile == 0L) {
            throw new IllegalStateException("The object is not initialized.");
        }
        return;
    }
    
    private void ensureOpenOrZipException() throws IOException {
        if (this.closeRequested) {
            throw new ZipException("ZipFile closed");
        }
        return;
    }
    
    private boolean startsWithLocHeader() {
        return this.locsig;
    }
    
    private static native long open(String p0, int p1, long p2, boolean p3) throws IOException;
    
    private static native int getTotal(long p0);
    
    private static native boolean startsWithLOC(long p0);
    
    private static native int read(long p0, long p1, long p2, byte[] p3, int p4, int p5);
    
    private static native long getEntryTime(long p0);
    
    private static native long getEntryCrc(long p0);
    
    private static native long getEntryCSize(long p0);
    
    private static native long getEntrySize(long p0);
    
    private static native int getEntryMethod(long p0);
    
    private static native int getEntryFlag(long p0);
    
    private static native byte[] getCommentBytes(long p0);
    
    private static native byte[] getEntryBytes(long p0, int p1);
    
    private static native String getZipMessage(long p0);
    
    static void access$100(ZipFile zipFile, Inflater inflater) {
        zipFile.releaseInflater(inflater);
    }
    
    static void access$200(ZipFile zipFile) {
        zipFile.ensureOpen();
    }
    
    static long access$500(long n, int n2) {
        return getNextEntry(n, n2);
    }
    
    static String access$700(long n) {
        return getZipMessage(n);
    }
    
    static ZipEntry access$900(ZipFile zipFile, String s, long n) {
        return zipFile.getZipEntry(s, n);
    }
    
    static void access$1000(long n, long n2) {
        freeEntry(n, n2);
    }
    
    static long access$1100(long n) {
        return getEntryCSize(n);
    }
    
    static long access$1200(long n) {
        return getEntrySize(n);
    }
    
    static void access$1300(ZipFile zipFile) throws IOException {
        zipFile.ensureOpenOrZipException();
    }
    
    static int access$1400(long n, long n2, long n3, byte[] array, int n4, int n5) {
        return read(n, n2, n3, array, n4, n5);
    }
    
    static boolean access$1500(ZipFile zipFile) {
        return zipFile.startsWithLocHeader();
    }
    
    static {
        initIDs();
        String savedProperty = VM.getSavedProperty("sun.zip.disableMemoryMapping");
        usemmap = (savedProperty == null || (savedProperty.length() != 0 && !savedProperty.equalsIgnoreCase("true")));
        String savedProperty2 = VM.getSavedProperty("jdk.util.zip.ensureTrailingSlash");
        ensuretrailingslash = (savedProperty2 == null || !savedProperty2.equalsIgnoreCase("false"));
        SharedSecrets.setJavaUtilZipFileAccess(new JavaUtilZipFileAccess() {
            public boolean startsWithLocHeader(ZipFile zipFile) {
                return ZipFile.access$1500(zipFile);
            }
        });
    }
    
    private class ZipFileInflaterInputStream extends InflaterInputStream
    {
        private volatile boolean closeRequested;
        private boolean eof;
        private final ZipFile.ZipFileInputStream zfin;
        
        ZipFileInflaterInputStream(ZipFile.ZipFileInputStream zfin, Inflater inflater, int n) {
            super(zfin, inflater, n);
            this.closeRequested = false;
            this.eof = false;
            this.zfin = zfin;
        }
        
        @Override
        public void close() throws IOException {
            if (this.closeRequested) {
                return;
            }
            this.closeRequested = true;
            super.close();
            Inflater inflater;
            synchronized (ZipFile.this.streams) {
                inflater = (Inflater)ZipFile.this.streams.remove(this);
            }
            if (inflater != null) {
                ZipFile.access$100(ZipFile.this, inflater);
            }
            return;
        }
        
        @Override
        protected void fill() throws IOException {
            if (this.eof) {
                throw new EOFException("Unexpected end of ZLIB input stream");
            }
            this.len = this.in.read(this.buf, 0, this.buf.length);
            if (this.len == -1) {
                this.buf[0] = 0;
                this.len = 1;
                this.eof = true;
            }
            this.inf.setInput(this.buf, 0, this.len);
            return;
        }
        
        @Override
        public int available() throws IOException {
            if (this.closeRequested) {
                return 0;
            }
            long n = this.zfin.size() - this.inf.getBytesWritten();
            return (n > 2147483647L) ? Integer.MAX_VALUE : ((int)n);
        }
        
        @Override
        protected void finalize() throws Throwable {
            this.close();
        }
    }
    
    private class ZipEntryIterator implements Enumeration<ZipEntry>, Iterator<ZipEntry>
    {
        private int i;
        
        public ZipEntryIterator() {
            this.i = 0;
            ZipFile.access$200(ZipFile.this);
        }
        
        @Override
        public boolean hasMoreElements() {
            return this.hasNext();
        }
        
        @Override
        public boolean hasNext() {
            synchronized (ZipFile.this) {
                ZipFile.access$200(ZipFile.this);
                return this.i < ZipFile.this.total;
            }
        }
        
        @Override
        public ZipEntry nextElement() {
            return this.next();
        }
        
        @Override
        public ZipEntry next() {
            synchronized (ZipFile.this) {
                ZipFile.access$200(ZipFile.this);
                if (this.i >= ZipFile.this.total) {
                    throw new NoSuchElementException();
                }
                long access$500 = ZipFile.access$500(ZipFile.this.jzfile, this.i++);
                if (access$500 == 0L) {
                    String access$502;
                    if (ZipFile.this.closeRequested) {
                        access$502 = "ZipFile concurrently closed";
                    }
                    else {
                        access$502 = ZipFile.access$700(ZipFile.this.jzfile);
                    }
                    throw new ZipError("jzentry == 0,\n jzfile = " + ZipFile.this.jzfile + ",\n total = " + ZipFile.this.total + ",\n name = " + ZipFile.this.name + ",\n i = " + this.i + ",\n message = " + access$502);
                }
                else {
                    ZipEntry access$501 = ZipFile.access$900(ZipFile.this, null, access$500);
                    ZipFile.access$1000(ZipFile.this.jzfile, access$500);
                    return access$501;
                }
            }
        }
    }
    
    private class ZipFileInputStream extends InputStream
    {
        private volatile boolean zfisCloseRequested;
        protected long jzentry;
        private long pos;
        protected long rem;
        protected long size;
        
        ZipFileInputStream(long jzentry) {
            this.zfisCloseRequested = false;
            this.pos = 0L;
            this.rem = ZipFile.access$1100(jzentry);
            this.size = ZipFile.access$1200(jzentry);
            this.jzentry = jzentry;
        }
        
        @Override
        public int read(byte[] array, int n, int access$1400) throws IOException {
            synchronized (ZipFile.this) {
                long rem = this.rem;
                long pos = this.pos;
                if (rem == 0L) {
                    return -1;
                }
                if (access$1400 <= 0) {
                    return 0;
                }
                if ((long)access$1400 > rem) {
                    access$1400 = (int)rem;
                }
                ZipFile.access$1300(ZipFile.this);
                access$1400 = ZipFile.access$1400(ZipFile.this.jzfile, this.jzentry, pos, array, n, access$1400);
                if (access$1400 > 0) {
                    this.pos = pos + (long)access$1400;
                    this.rem = rem - (long)access$1400;
                }
            }
            if (this.rem == 0L) {
                this.close();
            }
            return access$1400;
        }
        
        @Override
        public int read() throws IOException {
            byte[] array = { 0 };
            if (this.read(array, 0, 1) == 1) {
                return array[0] & 0xFF;
            }
            return -1;
        }
        
        @Override
        public long skip(long rem) {
            if (rem > this.rem) {
                rem = this.rem;
            }
            this.pos += rem;
            this.rem -= rem;
            if (this.rem == 0L) {
                this.close();
            }
            return rem;
        }
        
        @Override
        public int available() {
            return (this.rem > 2147483647L) ? Integer.MAX_VALUE : ((int)this.rem);
        }
        
        public long size() {
            return this.size;
        }
        
        @Override
        public void close() {
            if (this.zfisCloseRequested) {
                return;
            }
            this.zfisCloseRequested = true;
            this.rem = 0L;
            synchronized (ZipFile.this) {
                if (this.jzentry != 0L && ZipFile.this.jzfile != 0L) {
                    ZipFile.access$1000(ZipFile.this.jzfile, this.jzentry);
                    this.jzentry = 0L;
                }
            }
            synchronized (ZipFile.this.streams) {
                ZipFile.this.streams.remove(this);
            }
            return;
        }
        
        @Override
        protected void finalize() {
            this.close();
        }
    }
}
